@@ -1,7 +1,7 @@
 use core::fmt;
 use std::{
-    collections::HashMap,
-    io::{BufRead, Cursor, Seek, SeekFrom},
+    collections::{HashMap, HashSet},
+    io::{BufReader, Cursor, Read, Seek, SeekFrom},
     sync::{Arc, Mutex, MutexGuard},
 };
 
@@ -12,13 +12,13 @@ use nalgebra::Vector2;
 use crate::{
     channel::{AcquisitionChannel, ChannelIdentifier},
     convert::DCMLocation,
-    error::MCDError,
+    error::{MCDError, Result},
     mcd::AcquisitionXML,
     transform::AffineTransform,
     BoundingBox, ChannelImage, OnSlide, OpticalImage, Print, Region,
 };
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub enum DataFormat {
     Float,
 }
@@ -42,6 +42,19 @@ pub enum AcquisitionIdentifier {
     Description(String),
 }
 
+impl AcquisitionIdentifier {
+    /// Create an acquisition identifier based on a description
+    pub fn description(description: &str) -> Self {
+        AcquisitionIdentifier::Description(description.into())
+    }
+}
+
+impl From<&str> for AcquisitionIdentifier {
+    fn from(description: &str) -> Self {
+        Self::Description(description.into())
+    }
+}
+
 impl fmt::Display for AcquisitionIdentifier {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
@@ -58,7 +71,7 @@ impl fmt::Display for AcquisitionIdentifier {
     }
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone, Copy)]
 pub enum ProfilingType {
     Global,
 }
@@ -70,7 +83,7 @@ pub trait Acquisitions {
     fn channels(&self) -> Vec<&AcquisitionChannel>;
 }
 
-impl<T: BufRead + Seek> Acquisitions for Vec<&Acquisition<T>> {
+impl<R> Acquisitions for Vec<&Acquisition<R>> {
     fn channels(&self) -> Vec<&AcquisitionChannel> {
         let mut channels = HashMap::new();
 
@@ -95,8 +108,8 @@ impl<T: BufRead + Seek> Acquisitions for Vec<&Acquisition<T>> {
 
 /// Acquisition represents a single region analysed by IMC.
 #[derive(Debug)]
-pub struct Acquisition<T: BufRead + Seek> {
-    pub(crate) reader: Option<Arc<Mutex<T>>>,
+pub struct Acquisition<R> {
+    pub(crate) reader: Option<Arc<Mutex<BufReader<R>>>>,
     pub(crate) dcm_location: Option<DCMLocation>,
 
     id: u16,
@@ -135,14 +148,55 @@ pub struct Acquisition<T: BufRead + Seek> {
     channels: Vec<AcquisitionChannel>,
 }
 
-pub struct SpectrumIterator<'a, T: BufRead + Seek> {
-    acquisition: &'a Acquisition<T>,
-    reader: MutexGuard<'a, T>,
+impl<R> Clone for Acquisition<R> {
+    fn clone(&self) -> Self {
+        Self {
+            reader: self.reader.clone(),
+            dcm_location: self.dcm_location.clone(),
+            id: self.id,
+            description: self.description.clone(),
+            ablation_power: self.ablation_power,
+            ablation_distance_between_shots_x: self.ablation_distance_between_shots_x,
+            ablation_distance_between_shots_y: self.ablation_distance_between_shots_y,
+            ablation_frequency: self.ablation_frequency,
+            acquisition_roi_id: self.acquisition_roi_id,
+            order_number: self.order_number,
+            signal_type: self.signal_type.clone(),
+            dual_count_start: self.dual_count_start.clone(),
+            data_start_offset: self.data_start_offset,
+            data_end_offset: self.data_end_offset,
+            start_timestamp: self.start_timestamp.clone(),
+            end_timestamp: self.end_timestamp.clone(),
+            after_ablation_image_start_offset: self.after_ablation_image_start_offset,
+            after_ablation_image_end_offset: self.after_ablation_image_end_offset,
+            before_ablation_image_start_offset: self.before_ablation_image_start_offset,
+            before_ablation_image_end_offset: self.before_ablation_image_end_offset,
+            roi_start_x_pos_um: self.roi_start_x_pos_um,
+            roi_start_y_pos_um: self.roi_start_y_pos_um,
+            roi_end_x_pos_um: self.roi_end_x_pos_um,
+            roi_end_y_pos_um: self.roi_end_y_pos_um,
+            movement_type: self.movement_type.clone(),
+            segment_data_format: self.segment_data_format.clone(),
+            value_bytes: self.value_bytes,
+            max_x: self.max_x,
+            max_y: self.max_y,
+            plume_start: self.plume_start,
+            plume_end: self.plume_end,
+            template: self.template.clone(),
+            profiling_type: self.profiling_type,
+            channels: self.channels.clone(),
+        }
+    }
+}
+
+pub struct SpectrumIterator<'a, R> {
+    acquisition: &'a Acquisition<R>,
+    reader: MutexGuard<'a, BufReader<R>>,
     buffer: Vec<u8>,
 }
 
-impl<'a, T: BufRead + Seek> SpectrumIterator<'a, T> {
-    fn new(acquisition: &'a Acquisition<T>) -> Self {
+impl<'a, R: Seek> SpectrumIterator<'a, R> {
+    fn new(acquisition: &'a Acquisition<R>) -> Self {
         let mut reader = acquisition.reader.as_ref().unwrap().lock().unwrap();
 
         let offset = acquisition.data_start_offset as u64;
@@ -158,7 +212,7 @@ impl<'a, T: BufRead + Seek> SpectrumIterator<'a, T> {
     }
 }
 
-impl<'a, T: BufRead + Seek> Iterator for SpectrumIterator<'a, T> {
+impl<'a, R: Read + Seek> Iterator for SpectrumIterator<'a, R> {
     type Item = Vec<f32>;
 
     fn next(&mut self) -> Option<Vec<f32>> {
@@ -182,7 +236,7 @@ impl<'a, T: BufRead + Seek> Iterator for SpectrumIterator<'a, T> {
     }
 }
 
-impl<T: BufRead + Seek> Acquisition<T> {
+impl<R> Acquisition<R> {
     /// Returns the ID associated with the acquisition
     pub fn id(&self) -> u16 {
         self.id
@@ -240,7 +294,7 @@ impl<T: BufRead + Seek> Acquisition<T> {
     }*/
 
     /// Returns the optical image of the acquisition region prior to ablation
-    pub fn before_ablation_image(&self) -> OpticalImage<T> {
+    pub fn before_ablation_image(&self) -> OpticalImage<R> {
         OpticalImage {
             reader: self.reader.as_ref().unwrap().clone(),
             start_offset: self.before_ablation_image_start_offset,
@@ -257,7 +311,7 @@ impl<T: BufRead + Seek> Acquisition<T> {
     }
 
     /// Returns the optical image of the acquisition region after ablation
-    pub fn after_ablation_image(&self) -> OpticalImage<T> {
+    pub fn after_ablation_image(&self) -> OpticalImage<R> {
         OpticalImage {
             reader: self.reader.as_ref().unwrap().clone(),
             start_offset: self.after_ablation_image_start_offset,
@@ -284,7 +338,7 @@ impl<T: BufRead + Seek> Acquisition<T> {
             * self.value_bytes as usize;
         let measured_size: usize = self.data_end_offset as usize - self.data_start_offset as usize;
 
-        println!("Expected: {} | Measured: {}", expected_size, measured_size);
+        // println!("Expected: {} | Measured: {}", expected_size, measured_size);
 
         expected_size == measured_size
     }
@@ -354,41 +408,6 @@ impl<T: BufRead + Seek> Acquisition<T> {
         false
     }
 
-    /// Provides an iterator over all spectra (each pixel) within the acquisition
-    pub fn spectra(&self) -> SpectrumIterator<T> {
-        SpectrumIterator::new(self)
-    }
-
-    /// Returns a spectrum at the specified (x, y) coordinate
-    pub fn spectrum(&self, x: usize, y: usize) -> Result<Vec<f32>, MCDError> {
-        let index = y * self.max_x as usize + x;
-
-        if index >= self.num_spectra() {
-            return Err(MCDError::InvalidIndex {
-                index,
-                num_spectra: self.num_spectra(),
-            });
-        }
-
-        let offset = self.data_start_offset as u64
-            + (index * self.channels.len() * self.value_bytes as usize) as u64;
-
-        let mut spectrum = Vec::with_capacity(self.channels.len());
-        let mut reader = self.reader.as_ref().unwrap().lock().unwrap();
-
-        // TODO: Handle this properly without unwrapping
-        reader.seek(SeekFrom::Start(offset)).unwrap();
-
-        let mut buffer = [0u8; 4];
-        for _i in 0..self.channels.len() {
-            reader.read_exact(&mut buffer)?;
-            let float = f32::from_le_bytes(buffer);
-            spectrum.push(float);
-        }
-
-        Ok(spectrum)
-    }
-
     /// Returns the size of a single spectrum in bytes
     #[inline]
     pub fn spectrum_size(&self) -> usize {
@@ -402,36 +421,41 @@ impl<T: BufRead + Seek> Acquisition<T> {
 
         measured_size / self.spectrum_size()
     }
+}
 
+impl<R: Read + Seek> Acquisition<R> {
     /// Returns the ChannelImage for the channel matching the `ChannelIdentifier`. This contains the intensities of the channel
     /// for each detected pixel, the number of valid pixels and the width and height of the image.
-    pub fn channel_image(
+    pub fn channel_image<C: Into<ChannelIdentifier>>(
         &self,
-        identifier: &ChannelIdentifier,
+        identifier: C,
         region: Option<Region>,
-    ) -> Result<ChannelImage, MCDError> {
-        self.channel_images(&[identifier.clone()], region)
-            .map(|mut data| data.drain(..).last().unwrap())
+    ) -> Result<ChannelImage> {
+        Ok(self
+            .channel_images(&[identifier.into()], region)?
+            .drain(..)
+            .last()
+            .expect("A channel image should always be returned, as we always pass one identifier"))
     }
 
     /// Returns array of ChannelImages for the channels matching the `ChannelIdentifier`s. This contains the intensities of the channel
     /// for each detected pixel, the number of valid pixels and the width and height of the image.
-    pub fn channel_images(
+    pub fn channel_images<C: AsRef<ChannelIdentifier>>(
         &self,
-        identifiers: &[ChannelIdentifier],
+        identifiers: &[C],
         region: Option<Region>,
-    ) -> Result<Vec<ChannelImage>, MCDError> {
+    ) -> Result<Vec<ChannelImage>> {
         // println!("Searching identifiers: {:?}", identifiers);
         // println!("Searching from channels: {:?}", self.channels());
 
-        let channels: Option<Vec<_>> = identifiers
+        let channels: Vec<_> = identifiers
             .iter()
-            .map(|identifier| self.channel(identifier))
-            .collect();
-
-        let channels = channels.ok_or(MCDError::NoSuchChannel {
-            acquisition: AcquisitionIdentifier::Id(self.id),
-        })?;
+            .map(|identifier| {
+                self.channel(identifier).ok_or(MCDError::InvalidChannel {
+                    channel: identifier.as_ref().clone(),
+                })
+            })
+            .collect::<Result<Vec<_>>>()?;
 
         let order_numbers: Vec<_> = channels
             .iter()
@@ -454,79 +478,71 @@ impl<T: BufRead + Seek> Acquisition<T> {
         let valid_region_row = (region.y + region.height).min(last_row as u32);
         let valid_region_col = (region.x + region.width).min(last_col as u32);
 
-        let valid_pixels =
-            ((valid_region_row - region.y - 1) * region.width) + (valid_region_col - region.x);
-
-        // println!(
-        //     "Valid pixels: {} | {} {} | {} {} | {}",
-        //     valid_pixels,
-        //     valid_region_col,
-        //     valid_region_row,
-        //     self.width(),
-        //     self.height(),
-        //     self.num_spectra()
-        // );
-
-        // let mut data = vec![0.0; valid_pixels as usize];
-        // let mut min_value = f32::MAX;
-        // let mut max_value = f32::MIN;
-
-        //println!("DCM Location: {:?}", self.dcm_location);
-
-        if let Some(data_location) = &self.dcm_location {
-            let mut data = data_location.read_channels(&order_numbers, &region)?;
-
-            let images: Vec<_> = data
-                .drain(..)
-                //.zip(channels.iter())
-                .map(|data| {
-                    let mut min_value = f32::MAX;
-                    let mut max_value = f32::MIN;
-
-                    for &data_point in data.iter() {
-                        if data_point < min_value {
-                            min_value = data_point;
-                        }
-                        if data_point > max_value {
-                            max_value = data_point;
-                        }
-                    }
-
-                    ChannelImage {
-                        region,
-                        range: (min_value, max_value),
-                        valid_pixels: valid_pixels as usize,
-                        data,
-                    }
-                })
-                .collect();
-
-            Ok(images)
+        // println!("{} / {}", valid_region_row, region.y);
+        let valid_pixels = if region.y >= valid_region_row {
+            (valid_region_row - 1) * region.width + valid_region_col
         } else {
-            todo!();
-            // let offset = self.data_start_offset as u64
-            //     + channel.order_number() as u64 * self.value_bytes as u64;
+            ((valid_region_row - region.y - 1) * region.width) + (valid_region_col - region.x)
+        };
 
-            // let mut reader = self.reader.as_ref().unwrap().lock().unwrap();
-            // // TODO: Currently this only works for f32
-            // let mut buf = [0; 4];
+        let mut data = if let Some(data_location) = &self.dcm_location {
+            data_location.read_channels(&order_numbers, &region)?
+        } else {
+            let mut data: Vec<Vec<f32>> =
+                vec![
+                    Vec::with_capacity((region.width * region.height) as usize);
+                    identifiers.len()
+                ];
 
-            // reader.seek(SeekFrom::Start(offset))?;
-            // for data_point in data.iter_mut() {
-            //     reader.read_exact(&mut buf)?;
+            let order_hash: HashSet<usize> = HashSet::from_iter(order_numbers.iter().copied());
 
-            //     *data_point = f32::from_le_bytes(buf);
+            for y in region.y..(region.y + region.height) {
+                for x in region.x..(region.x + region.width) {
+                    for (channel_index, intensity) in self
+                        .spectrum(x, y)?
+                        .drain(..)
+                        .enumerate()
+                        .filter(|(index, _intensity)| order_hash.contains(index))
+                        .map(|(_, intensity)| intensity)
+                        .enumerate()
+                    {
+                        data[channel_index].push(intensity);
+                    }
+                }
+            }
 
-            //     if *data_point < min_value {
-            //         min_value = *data_point;
-            //     }
-            //     if *data_point > max_value {
-            //         max_value = *data_point;
-            //     }
+            data
+        };
 
-            //     reader.seek(SeekFrom::Current(self.spectrum_size() as i64 - 4))?;
-            // }
-        }
+        let images: Vec<_> = data
+            .drain(..)
+            .zip(channels.iter())
+            .map(|(data, channel)| {
+                let mut min_value = f32::MAX;
+                let mut max_value = f32::MIN;
+
+                for &data_point in data.iter() {
+                    if data_point < min_value {
+                        min_value = data_point;
+                    }
+                    if data_point > max_value {
+                        max_value = data_point;
+                    }
+                }
+
+                ChannelImage {
+                    region,
+                    acquisition_id: channel.acquisition_id(),
+                    name: channel.name().to_string(),
+                    label: channel.label().to_string(),
+                    range: (min_value, max_value),
+                    valid_pixels: valid_pixels as usize,
+                    data,
+                }
+            })
+            .collect();
+
+        Ok(images)
 
         // Ok(ChannelImage {
         //     region,
@@ -537,14 +553,13 @@ impl<T: BufRead + Seek> Acquisition<T> {
     }
 
     /// Returns the channel which matches the given identifier, or None if no match found
-    pub fn channel(&self, identifier: &ChannelIdentifier) -> Option<&AcquisitionChannel> {
-        for channel in &self.channels {
-            if channel.is(identifier) {
-                return Some(channel);
-            }
-        }
-
-        None
+    pub fn channel<C: AsRef<ChannelIdentifier>>(
+        &self,
+        identifier: C,
+    ) -> Option<&AcquisitionChannel> {
+        self.channels
+            .iter()
+            .find(|&channel| channel.is(identifier.as_ref()))
     }
 
     // pub fn channel_index(&self, identifier: &ChannelIdentifier) -> Option<usize> {
@@ -570,7 +585,41 @@ impl<T: BufRead + Seek> Acquisition<T> {
     }
 }
 
-impl<T: Seek + BufRead> OnSlide for Acquisition<T> {
+impl<R: Read + Seek> Acquisition<R> {
+    /// Provides an iterator over all spectra (each pixel) within the acquisition
+    pub fn spectra(&self) -> SpectrumIterator<R> {
+        SpectrumIterator::new(self)
+    }
+
+    /// Returns a spectrum at the specified (x, y) coordinate
+    pub fn spectrum(&self, x: u32, y: u32) -> Result<Vec<f32>> {
+        let index = y as usize * self.max_x as usize + x as usize;
+
+        if index >= self.num_spectra() {
+            return Err(MCDError::InvalidIndex {
+                index,
+                num_spectra: self.num_spectra(),
+            });
+        }
+
+        let offset = self.data_start_offset as u64
+            + (index * self.channels.len() * self.value_bytes as usize) as u64;
+
+        let mut spectrum = Vec::with_capacity(self.channels.len());
+        let mut reader = self.reader.as_ref().unwrap().lock().unwrap();
+
+        // TODO: Handle this properly without unwrapping
+        reader.seek(SeekFrom::Start(offset))?;
+
+        for i in 0..self.channels.len() {
+            spectrum.push(reader.read_f32::<LittleEndian>()?);
+        }
+
+        Ok(spectrum)
+    }
+}
+
+impl<R> OnSlide for Acquisition<R> {
     /// Returns the affine transformation from pixel coordinates within the acquisition to to the slide coordinates (μm)
     fn to_slide_transform(&self) -> AffineTransform<f64> {
         let mut moving_points = Vec::new();
@@ -636,7 +685,7 @@ impl<T: Seek + BufRead> OnSlide for Acquisition<T> {
 }
 
 #[rustfmt::skip]
-impl<T: Seek + BufRead> Print for Acquisition<T> {
+impl<R> Print for Acquisition<R> {
     fn print<W: fmt::Write + ?Sized>(&self, writer: &mut W, indent: usize) -> fmt::Result {
         write!(writer, "{:indent$}", "", indent = indent)?;
         writeln!(writer, "{:-^1$}", "Acquisition", 48)?;
@@ -773,13 +822,13 @@ impl<T: Seek + BufRead> Print for Acquisition<T> {
     }
 }
 
-impl<T: Seek + BufRead> fmt::Display for Acquisition<T> {
+impl<R> fmt::Display for Acquisition<R> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         self.print(f, 0)
     }
 }
 
-impl<T: Seek + BufRead> From<AcquisitionXML> for Acquisition<T> {
+impl<R> From<AcquisitionXML> for Acquisition<R> {
     fn from(acquisition: AcquisitionXML) -> Self {
         Acquisition {
             reader: None,
