@@ -11,8 +11,40 @@ pub enum ChannelIdentifier {
     Label(String),
 }
 
+impl ChannelIdentifier {
+    pub fn label(label: &str) -> Self {
+        Self::Label(label.into())
+    }
+
+    pub fn name(name: &str) -> Self {
+        Self::Name(name.into())
+    }
+
+    pub fn order(order: i16) -> Self {
+        Self::Order(order)
+    }
+}
+
+impl AsRef<ChannelIdentifier> for ChannelIdentifier {
+    fn as_ref(&self) -> &ChannelIdentifier {
+        self
+    }
+}
+
+impl From<&ChannelIdentifier> for ChannelIdentifier {
+    fn from(identifier: &ChannelIdentifier) -> Self {
+        identifier.clone()
+    }
+}
+
+impl From<&AcquisitionChannel> for ChannelIdentifier {
+    fn from(acq_channel: &AcquisitionChannel) -> Self {
+        ChannelIdentifier::Name(acq_channel.channel_name.to_owned())
+    }
+}
+
 /// AcquisitionChannel represents a single channel acquired, forming part of an acquisition
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct AcquisitionChannel {
     id: u16,
     channel_name: String,
