@@ -1,11 +1,12 @@
 use std::{
     collections::HashMap,
     fs::File,
-    io::{BufRead, BufReader, BufWriter, Cursor, Read, Seek, SeekFrom, Write},
+    io::{BufReader, Cursor, Read, Seek, SeekFrom, Write},
     sync::{Arc, Mutex},
 };
 
 use byteorder::{LittleEndian, ReadBytesExt, WriteBytesExt};
+use rayon::prelude::{ParallelDrainRange, ParallelIterator};
 
 use crate::{error::MCDError, Acquisition, Region, MCD};
 
@@ -23,7 +24,7 @@ struct AcquisitionOffset {
 // number of acquisitions (u8)
 // offsets for each acquisition ((u16, u64, u8))
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 struct AcquisitionDetails {
     width: u32,
     height: u32,
@@ -35,7 +36,7 @@ struct AcquisitionDetails {
 }
 
 impl AcquisitionDetails {
-    fn from<T: BufRead + Seek>(acquisition: &Acquisition<T>, chunk_size: u32) -> Self {
+    fn from<R>(acquisition: &Acquisition<R>, chunk_size: u32) -> Self {
         AcquisitionDetails {
             width: acquisition.width() as u32,
             height: acquisition.height() as u32,
@@ -66,14 +67,14 @@ impl AcquisitionDetails {
     }
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 struct ChannelChunk {
     num_intensities: u64,
     offset: u64,
     length: u64,
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 struct PixelChunk {
     channels: Vec<ChannelChunk>,
 }
@@ -86,11 +87,12 @@ impl PixelChunk {
     }
 }
 
-pub fn convert<T: BufRead + Seek>(mcd: &MCD<T>) -> Result<(), MCDError> {
+pub fn convert<R: Read + Seek, W: Write + Seek>(
+    mcd: &MCD<R>,
+    mut dcm_file: W,
+) -> Result<(), MCDError> {
     //let mut acquisition_offsets = HashMap::new();
     //println!("Opening {:?} for writing", mcd.dcm_file());
-    let dcm_file = std::fs::File::create(mcd.dcm_file())?;
-    let mut dcm_file = BufWriter::new(dcm_file);
 
     let mut num_acquisitions = 0;
 
@@ -102,7 +104,7 @@ pub fn convert<T: BufRead + Seek>(mcd: &MCD<T>) -> Result<(), MCDError> {
 
     //println!("Writing {} acquisitions.", num_acquisitions);
 
-    let chunk_size = 128;
+    let chunk_size = 256;
 
     //dcm_file.write_u8(chunk_size as u8)?;
 
@@ -117,12 +119,12 @@ pub fn convert<T: BufRead + Seek>(mcd: &MCD<T>) -> Result<(), MCDError> {
             for acquisition in panorama.acquisitions() {
                 let mut acq_details = AcquisitionDetails::from(acquisition, chunk_size);
 
-                println!(
-                    "[{}] Total # chunks: ({}, {})",
-                    acquisition.description(),
-                    acq_details.num_chunks_x(),
-                    acq_details.num_chunks_y()
-                );
+                // println!(
+                //     "[{}] Total # chunks: ({}, {})",
+                //     acquisition.description(),
+                //     acq_details.num_chunks_x(),
+                //     acq_details.num_chunks_y()
+                // );
 
                 for y_chunk in 0..acq_details.num_chunks_y() {
                     for x_chunk in 0..acq_details.num_chunks_x() {
@@ -147,7 +149,7 @@ pub fn convert<T: BufRead + Seek>(mcd: &MCD<T>) -> Result<(), MCDError> {
 
                         for y in y_start..y_stop {
                             for x in x_start..x_stop {
-                                let spectrum = match acquisition.spectrum(x as usize, y as usize) {
+                                let spectrum = match acquisition.spectrum(x, y) {
                                     Ok(spectrum) => spectrum,
                                     Err(MCDError::InvalidIndex {
                                         index: _,
@@ -160,25 +162,53 @@ pub fn convert<T: BufRead + Seek>(mcd: &MCD<T>) -> Result<(), MCDError> {
                                     }
                                 };
 
-                                for (index, intensity) in spectrum.iter().enumerate() {
-                                    channel_chunks[index].push(*intensity);
+                                for (channel_chunk, intensity) in
+                                    channel_chunks.iter_mut().zip(spectrum.iter())
+                                {
+                                    channel_chunk.push(*intensity);
                                 }
                             }
                         }
 
                         let mut pixel_chunk = PixelChunk::new();
 
-                        for channel_chunk in channel_chunks {
-                            let num_intensities = channel_chunk.len();
+                        let compressed_chunks = channel_chunks
+                            .par_drain(..)
+                            .map(|channel_chunk| {
+                                let num_intensities = channel_chunk.len();
 
-                            let mut buf: Vec<u8> = Vec::with_capacity(channel_chunk.len() * 4);
+                                let mut buf: Vec<u8> = Vec::with_capacity(channel_chunk.len() * 4);
 
-                            for intensity in channel_chunk {
-                                buf.write_f32::<LittleEndian>(intensity)?;
-                            }
+                                for intensity in channel_chunk {
+                                    buf.write_f32::<LittleEndian>(intensity)?;
+                                }
 
-                            let compressed = lz4_flex::compress(&buf);
+                                Ok((num_intensities, lz4_flex::compress(&buf)))
+                            })
+                            .collect::<Result<Vec<_>, MCDError>>()?;
 
+                        // for channel_chunk in channel_chunks {
+                        //     let num_intensities = channel_chunk.len();
+
+                        //     let mut buf: Vec<u8> = Vec::with_capacity(channel_chunk.len() * 4);
+
+                        //     for intensity in channel_chunk {
+                        //         buf.write_f32::<LittleEndian>(intensity)?;
+                        //     }
+
+                        //     let compressed = lz4_flex::compress(&buf);
+
+                        //     let cur_location = dcm_file.seek(SeekFrom::Current(0))?;
+                        //     dcm_file.write_all(&compressed)?;
+                        //     let new_location = dcm_file.seek(SeekFrom::Current(0))?;
+
+                        //     pixel_chunk.channels.push(ChannelChunk {
+                        //         num_intensities: num_intensities as u64,
+                        //         offset: cur_location,
+                        //         length: new_location - cur_location,
+                        //     });
+                        // }
+                        for (num_intensities, compressed) in compressed_chunks {
                             let cur_location = dcm_file.seek(SeekFrom::Current(0))?;
                             dcm_file.write_all(&compressed)?;
                             let new_location = dcm_file.seek(SeekFrom::Current(0))?;
@@ -194,7 +224,7 @@ pub fn convert<T: BufRead + Seek>(mcd: &MCD<T>) -> Result<(), MCDError> {
                     }
                 }
 
-                let acquisition_index_location = dcm_file.seek(SeekFrom::Current(0)).unwrap();
+                let acquisition_index_location = dcm_file.seek(SeekFrom::Current(0))?;
                 acquisition_index.push((acquisition.id(), acquisition_index_location));
 
                 dcm_file.write_acquisition_details(&acq_details)?;
@@ -313,9 +343,9 @@ impl<T: WriteBytesExt> WriteDCM for T {
     }
 }
 
-pub fn open<T: BufRead + Seek>(mcd: &mut MCD<T>) -> Result<(), MCDError> {
+pub fn open(mcd: &mut MCD<File>) -> Result<(), MCDError> {
     //println!("Opening {:?} for reading", mcd.dcm_file());
-    let dcm_file = std::fs::File::open(mcd.dcm_file())?;
+    let dcm_file = std::fs::File::open(mcd.dcm_file().ok_or(MCDError::LocationNotSpecified)?)?;
     let dcm_file_arc = Arc::new(Mutex::new(BufReader::new(dcm_file)));
     let mut dcm_file = dcm_file_arc.lock().unwrap();
 
@@ -357,7 +387,7 @@ pub fn open<T: BufRead + Seek>(mcd: &mut MCD<T>) -> Result<(), MCDError> {
     Ok(())
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct DCMLocation {
     reader: Arc<Mutex<BufReader<File>>>,
     details: AcquisitionDetails,
@@ -398,6 +428,8 @@ impl DCMLocation {
                 let start_y = chunk_y * self.details.chunk_size;
                 let end_y = (start_y + self.details.chunk_size).min(self.details.acquired_height());
 
+                let chunk_width = end_x - start_x;
+
                 let chunk_index = (chunk_y * self.details.num_chunks_x()) + chunk_x;
 
                 let pixel_chunk = &self.details.chunks[chunk_index as usize];
@@ -417,8 +449,7 @@ impl DCMLocation {
 
                     for y in start_y..end_y {
                         if region.y > y {
-                            decompressed_data
-                                .seek(SeekFrom::Current(self.details.chunk_size as i64 * 4))?;
+                            decompressed_data.seek(SeekFrom::Current(chunk_width as i64 * 4))?;
 
                             continue;
                         }
@@ -450,8 +481,8 @@ impl DCMLocation {
                                 break;
                             }
 
-                            let intensity = decompressed_data.read_f32::<LittleEndian>()?;
                             let index = ((y - region.y) * region.width) + (x - region.x);
+                            let intensity = decompressed_data.read_f32::<LittleEndian>()?;
 
                             data[index as usize] = intensity;
                         }
