@@ -5,6 +5,7 @@ use quick_xml::events::Event;
 use crate::{
     acquisition::{DataFormat, ProfilingType},
     calibration::{Calibration, CalibrationChannel, CalibrationFinal, CalibrationParams},
+    error::MCDError,
     panorama::PanoramaType,
     slide::{SlideFiducialMarks, SlideProfile},
 };
@@ -131,22 +132,22 @@ pub enum ParserState {
     Finished,
 }
 
-pub struct MCDParser<T: Seek + BufRead> {
-    pub(crate) current_mcd: Option<MCD<T>>,
+pub struct MCDParser<R> {
+    pub(crate) current_mcd: Option<MCD<R>>,
 
     state: ParserState,
     sub_state: ParserState,
     //pub(super) history: Vec<String>,
-    errors: std::collections::VecDeque<String>,
+    errors: std::collections::VecDeque<MCDError>,
 
-    panoramas: HashMap<u16, Panorama<T>>,
+    panoramas: HashMap<u16, Panorama<R>>,
     calibration_finals: HashMap<u16, CalibrationFinal>,
     calibration_params: HashMap<u16, CalibrationParams>,
     calibration_channels: HashMap<u16, CalibrationChannel>,
     calibrations: HashMap<u16, Calibration>,
     slide_fiducal_marks: HashMap<u16, SlideFiducialMarks>,
     slide_profiles: HashMap<u16, SlideProfile>,
-    acquisitions: HashMap<u16, Acquisition<T>>,
+    acquisitions: HashMap<u16, Acquisition<R>>,
     acquisition_channels: Vec<AcquisitionChannel>,
     acquisition_rois: Vec<AcquisitionROI>,
 
@@ -166,8 +167,8 @@ pub struct MCDParser<T: Seek + BufRead> {
     current_roi_point: Option<ROIPoint>,
 }
 
-impl<T: Seek + BufRead> MCDParser<T> {
-    pub fn new(mcd: MCD<T>) -> MCDParser<T> {
+impl<R: Read + Seek> MCDParser<R> {
+    pub fn new(mcd: MCD<R>) -> MCDParser<R> {
         MCDParser {
             current_mcd: Some(mcd),
             state: ParserState::Start,
@@ -203,7 +204,7 @@ impl<T: Seek + BufRead> MCDParser<T> {
         }
     }
 
-    pub fn mcd(&mut self) -> MCD<T> {
+    pub fn mcd(&mut self) -> MCD<R> {
         let mut mcd = self
             .current_mcd
             .take()
@@ -282,17 +283,17 @@ impl<T: Seek + BufRead> MCDParser<T> {
         !self.errors.is_empty()
     }
     #[allow(dead_code)]
-    pub fn pop_error_front(&mut self) -> Option<String> {
+    pub fn pop_error_front(&mut self) -> Option<MCDError> {
         self.errors.pop_front()
     }
 
-    pub fn pop_error_back(&mut self) -> Option<String> {
+    pub fn pop_error_back(&mut self) -> Option<MCDError> {
         self.errors.pop_back()
     }
 
     pub fn process(&mut self, ev: Event) {
         match &ev {
-            Event::Start(e) | Event::Empty(e) => match e.local_name() {
+            Event::Start(e) | Event::Empty(e) => match e.local_name().as_ref() {
                 b"MCDSchema" => {
                     // TODO: get xmlns
                     //self.current_mcd = Some()
@@ -476,21 +477,23 @@ impl<T: Seek + BufRead> MCDParser<T> {
                 b"SlideYPosUm" => self.sub_state = ParserState::ProcessingSlideYPosUm,
                 b"PanoramaPixelXPos" => self.sub_state = ParserState::ProcessingPanoramaPixelXPos,
                 b"PanoramaPixelYPos" => self.sub_state = ParserState::ProcessingPanoramaPixelYPos,
-                _ => match std::str::from_utf8(e.local_name()) {
+                _ => match std::str::from_utf8(e.local_name().as_ref()) {
                     Ok(name) => {
-                        self.errors
-                            .push_back(format!("[Start] Unknown tag name: {}", name));
+                        self.errors.push_back(MCDError::UnknownTag {
+                            name: name.to_owned(),
+                        });
 
-                        panic!("[Start] Unknown tag name: {}", name);
+                        // panic!("[Start] Unknown tag name: {}", name);
                     }
                     Err(error) => {
-                        println!("Failed to convert tag name: {}", error);
+                        // println!("Failed to convert tag name: {}", error);
+                        self.errors.push_back(error.into());
 
                         self.state = ParserState::FatalError
                     }
                 },
             },
-            Event::End(e) => match e.local_name() {
+            Event::End(e) => match e.local_name().as_ref() {
                 b"Slide" => {
                     let slide = self.current_slide.take().unwrap();
                     self.current_mcd
@@ -584,13 +587,14 @@ impl<T: Seek + BufRead> MCDParser<T> {
                     self.state = ParserState::Processing
                 }
                 b"MCDSchema" => self.state = ParserState::Finished,
-                _ => match std::str::from_utf8(e.local_name()) {
+                _ => match std::str::from_utf8(e.local_name().into_inner()) {
                     Ok(name) => {
-                        self.errors
-                            .push_back(format!("[End] Unknown tag name: {}", name));
+                        self.errors.push_back(MCDError::UnknownTag {
+                            name: name.to_owned(),
+                        });
                     }
                     Err(error) => {
-                        println!("Failed to convert tag name: {}", error);
+                        self.errors.push_back(error.into());
 
                         self.state = ParserState::FatalError
                     }
@@ -601,8 +605,8 @@ impl<T: Seek + BufRead> MCDParser<T> {
                 match self.state {
                     ParserState::ProcessingSlide => {
                         let slide = self.current_slide.as_mut().unwrap();
-                        let unprocessed_text = &e.unescaped().unwrap();
-                        let text = std::str::from_utf8(unprocessed_text).unwrap();
+                        let unprocessed_text = e.unescape().unwrap();
+                        let text = std::str::from_utf8(unprocessed_text.as_bytes()).unwrap();
 
                         match self.sub_state {
                             ParserState::ProcessingID => slide.id = Some(text.parse().unwrap()),
@@ -655,8 +659,8 @@ impl<T: Seek + BufRead> MCDParser<T> {
 
                     ParserState::ProcessingPanorama => {
                         let panorama = self.current_panorama.as_mut().unwrap();
-                        let unprocessed_text = &e.unescaped().unwrap();
-                        let text = std::str::from_utf8(unprocessed_text).unwrap();
+                        let unprocessed_text = e.unescape().unwrap();
+                        let text = std::str::from_utf8(unprocessed_text.as_bytes()).unwrap();
 
                         match self.sub_state {
                             ParserState::ProcessingID => panorama.id = Some(text.parse().unwrap()),
@@ -703,7 +707,8 @@ impl<T: Seek + BufRead> MCDParser<T> {
                                 panorama.pixel_height = Some(text.parse().unwrap())
                             }
                             ParserState::ProcessingImageFormat => {
-                                match std::str::from_utf8(&e.unescaped().unwrap()).as_ref() {
+                                match std::str::from_utf8(e.unescape().unwrap().as_bytes()).as_ref()
+                                {
                                     Ok(&"PNG") => panorama.image_format = Some(ImageFormat::Png),
                                     Ok(_) => todo!(),
                                     Err(_) => todo!(),
@@ -730,6 +735,7 @@ impl<T: Seek + BufRead> MCDParser<T> {
                                 panorama.is_locked = Some(text.parse().unwrap())
                             }
                             ParserState::ProcessingRotationAngle => {
+                                println!("{:?}", text);
                                 panorama.rotation_angle = Some(text.parse().unwrap())
                             }
                             ParserState::Processing => {}
@@ -746,8 +752,8 @@ impl<T: Seek + BufRead> MCDParser<T> {
 
                     ParserState::ProcessingCalibrationFinal => {
                         let calibration_final = self.current_calibration_final.as_mut().unwrap();
-                        let unprocessed_text = &e.unescaped().unwrap();
-                        let text = std::str::from_utf8(unprocessed_text).unwrap();
+                        let unprocessed_text = e.unescape().unwrap();
+                        let text = std::str::from_utf8(unprocessed_text.as_bytes()).unwrap();
 
                         match self.sub_state {
                             ParserState::ProcessingID => {
@@ -809,8 +815,8 @@ impl<T: Seek + BufRead> MCDParser<T> {
 
                     ParserState::ProcessingCalibrationParams => {
                         let calibration_params = self.current_calibration_params.as_mut().unwrap();
-                        let unprocessed_text = &e.unescaped().unwrap();
-                        let text = std::str::from_utf8(unprocessed_text).unwrap();
+                        let unprocessed_text = e.unescape().unwrap();
+                        let text = std::str::from_utf8(unprocessed_text.as_bytes()).unwrap();
 
                         match self.sub_state {
                             ParserState::ProcessingCalibrationID => {
@@ -865,8 +871,8 @@ impl<T: Seek + BufRead> MCDParser<T> {
                     ParserState::ProcessingCalibrationChannel => {
                         let calibration_channel =
                             self.current_calibration_channel.as_mut().unwrap();
-                        let unprocessed_text = &e.unescaped().unwrap();
-                        let text = std::str::from_utf8(unprocessed_text).unwrap();
+                        let unprocessed_text = e.unescape().unwrap();
+                        let text = std::str::from_utf8(unprocessed_text.as_bytes()).unwrap();
 
                         match self.sub_state {
                             ParserState::ProcessingID => {
@@ -895,8 +901,8 @@ impl<T: Seek + BufRead> MCDParser<T> {
 
                     ParserState::ProcessingCalibration => {
                         let calibration = self.current_calibration.as_mut().unwrap();
-                        let unprocessed_text = &e.unescaped().unwrap();
-                        let text = std::str::from_utf8(unprocessed_text).unwrap();
+                        let unprocessed_text = e.unescape().unwrap();
+                        let text = std::str::from_utf8(unprocessed_text.as_bytes()).unwrap();
 
                         match self.sub_state {
                             ParserState::ProcessingID => {
@@ -923,8 +929,8 @@ impl<T: Seek + BufRead> MCDParser<T> {
                     ParserState::ProcessingSlideFiducialMarks => {
                         let slide_fiducal_marks =
                             self.current_slide_fiducial_marks.as_mut().unwrap();
-                        let unprocessed_text = &e.unescaped().unwrap();
-                        let text = std::str::from_utf8(unprocessed_text).unwrap();
+                        let unprocessed_text = e.unescape().unwrap();
+                        let text = std::str::from_utf8(unprocessed_text.as_bytes()).unwrap();
 
                         match self.sub_state {
                             ParserState::ProcessingID => {
@@ -953,8 +959,8 @@ impl<T: Seek + BufRead> MCDParser<T> {
 
                     ParserState::ProcessingSlideProfile => {
                         let slide_profile = self.current_slide_profile.as_mut().unwrap();
-                        let unprocessed_text = &e.unescaped().unwrap();
-                        let text = std::str::from_utf8(unprocessed_text).unwrap();
+                        let unprocessed_text = e.unescape().unwrap();
+                        let text = std::str::from_utf8(unprocessed_text.as_bytes()).unwrap();
 
                         match self.sub_state {
                             ParserState::ProcessingID => {
@@ -984,8 +990,8 @@ impl<T: Seek + BufRead> MCDParser<T> {
                     ParserState::ProcessingAcquisitionChannel => {
                         let acquisition_channel =
                             self.current_acquisition_channel.as_mut().unwrap();
-                        let unprocessed_text = &e.unescaped().unwrap();
-                        let text = std::str::from_utf8(unprocessed_text).unwrap();
+                        let unprocessed_text = e.unescape().unwrap();
+                        let text = std::str::from_utf8(unprocessed_text.as_bytes()).unwrap();
 
                         match self.sub_state {
                             ParserState::ProcessingID => {
@@ -1027,8 +1033,8 @@ impl<T: Seek + BufRead> MCDParser<T> {
 
                     ParserState::ProcessingAcquisition => {
                         let acquisition = self.current_acquisition.as_mut().unwrap();
-                        let unprocessed_text = &e.unescaped().unwrap();
-                        let text = std::str::from_utf8(unprocessed_text).unwrap();
+                        let unprocessed_text = e.unescape().unwrap();
+                        let text = std::str::from_utf8(unprocessed_text.as_bytes()).unwrap();
 
                         match self.sub_state {
                             ParserState::ProcessingID => {
@@ -1154,8 +1160,8 @@ impl<T: Seek + BufRead> MCDParser<T> {
 
                     ParserState::ProcessingAcquisitionROI => {
                         let acquisition_roi = self.current_acquisition_roi.as_mut().unwrap();
-                        let unprocessed_text = &e.unescaped().unwrap();
-                        let text = std::str::from_utf8(unprocessed_text).unwrap();
+                        let unprocessed_text = e.unescape().unwrap();
+                        let text = std::str::from_utf8(unprocessed_text.as_bytes()).unwrap();
 
                         match self.sub_state {
                             ParserState::ProcessingID => {
@@ -1189,8 +1195,8 @@ impl<T: Seek + BufRead> MCDParser<T> {
 
                     ParserState::ProcessingROIPoint => {
                         let roi_point = self.current_roi_point.as_mut().unwrap();
-                        let unprocessed_text = &e.unescaped().unwrap();
-                        let text = std::str::from_utf8(unprocessed_text).unwrap();
+                        let unprocessed_text = e.unescape().unwrap();
+                        let text = std::str::from_utf8(unprocessed_text.as_bytes()).unwrap();
 
                         match self.sub_state {
                             ParserState::ProcessingID => roi_point.id = Some(text.parse().unwrap()),
