@@ -16,13 +16,12 @@ use pyo3::prelude::*;
 use numpy::{IntoPyArray, PyArray3};
 
 use std::fs::File;
-use std::io::BufReader;
 use std::sync::Arc;
 
 /// Mcd represents an .mcd file
 #[pyclass]
 struct Mcd {
-    mcd: Arc<imc_rs::MCD<BufReader<File>>>,
+    mcd: Arc<imc_rs::MCD<File>>,
 }
 
 struct PyMcdError(MCDError);
@@ -41,36 +40,36 @@ impl From<PyMcdError> for PyErr {
 
 #[pymethods]
 impl Mcd {
-    /// Parse an .mcd file, returning an object providing access to IMC data and accompanying metadata
-    #[staticmethod]
-    pub fn parse(filename: &str) -> PyResult<Self> {
-        let file = match File::open(filename) {
-            Ok(file) => file,
-            Err(error) => return Err(PyErr::new::<exceptions::PyIOError, _>(error)),
-        };
-
-        let mcd = imc_rs::MCD::parse(BufReader::new(file), filename)?;
-
-        Ok(Mcd { mcd: Arc::new(mcd) })
-    }
-
-    /// Parse an .mcd file, generating a temporary file for fast channel image access if one is not present, and
+    /// Parse an .mcd file, returning an object providing access to IMC data and accompanying metadata. If with_dcm is set to true, then
+    /// a temporary file for fast channel image access if one is not present, and
     /// returning an object providing access to IMC data and accompanying metadata
     #[staticmethod]
-    pub fn parse_with_dcm(filename: &str) -> PyResult<Self> {
-        let file = match File::open(filename) {
-            Ok(file) => file,
-            Err(error) => return Err(PyErr::new::<exceptions::PyIOError, _>(error)),
-        };
+    pub fn parse(filename: &str, with_dcm: Option<bool>) -> PyResult<Self> {
+        let with_dcm = with_dcm.unwrap_or(true);
 
-        let mcd = match imc_rs::MCD::parse_with_dcm(BufReader::new(file), filename) {
-            Ok(mcd) => mcd,
+        let mcd = match MCD::from_path(filename) {
+            Ok(mcd) => {
+                if with_dcm {
+                    match mcd.with_dcm() {
+                        Ok(mcd) => mcd,
+                        Err(error) => return Err(PyMcdError::from(error).into()),
+                    }
+                } else {
+                    mcd
+                }
+            }
             Err(error) => return Err(PyMcdError::from(error).into()),
         };
 
         Ok(Mcd { mcd: Arc::new(mcd) })
     }
 
+    #[deprecated]
+    #[staticmethod]
+    pub fn parse_with_dcm(filename: &str) -> PyResult<Self> {
+        Self::parse(filename, None)
+    }
+
     /// Returns the number of slides in the .mcd data
     pub fn num_slides(&self) -> PyResult<usize> {
         Ok(self.mcd.slide_ids().len())
@@ -93,7 +92,7 @@ impl Mcd {
     pub fn xml(&self) -> PyResult<String> {
         match self.mcd.xml() {
             Ok(xml) => Ok(xml),
-            Err(error) => Err(PyErr::new::<exceptions::PyIOError, _>(error)),
+            Err(error) => Err(PyMcdError::from(error).into()),
         }
     }
 
@@ -198,13 +197,13 @@ impl AcquisitionChannel {
 
 #[pyclass]
 struct Slide {
-    mcd: Arc<MCD<BufReader<File>>>,
+    mcd: Arc<MCD<File>>,
 
     id: u16,
 }
 
 impl Slide {
-    fn get_slide(&self) -> &imc_rs::Slide<BufReader<File>> {
+    fn get_slide(&self) -> &imc_rs::Slide<File> {
         self.mcd.slide(self.id).expect("Should be valid slide id")
     }
 }
@@ -330,14 +329,14 @@ impl Slide {
 
 #[pyclass]
 struct Panorama {
-    mcd: Arc<MCD<BufReader<File>>>,
+    mcd: Arc<MCD<File>>,
 
     id: u16,
     slide_id: u16,
 }
 
 impl Panorama {
-    fn get_panorama(&self) -> &imc_rs::Panorama<BufReader<File>> {
+    fn get_panorama(&self) -> &imc_rs::Panorama<File> {
         self.mcd
             .slide(self.slide_id)
             .expect("Should be valid slide id")
@@ -377,7 +376,7 @@ impl Panorama {
 
 #[pyclass]
 struct Acquisition {
-    mcd: Arc<imc_rs::MCD<BufReader<File>>>,
+    mcd: Arc<imc_rs::MCD<File>>,
 
     id: u16,
     panorama_id: u16,
@@ -385,7 +384,7 @@ struct Acquisition {
 }
 
 impl Acquisition {
-    fn get_acquisition(&self) -> &imc_rs::Acquisition<BufReader<File>> {
+    fn get_acquisition(&self) -> &imc_rs::Acquisition<File> {
         self.mcd
             .slide(self.slide_id)
             .expect("Should be valid slide id")
@@ -410,6 +409,18 @@ impl Acquisition {
         Ok(self.slide_id)
     }
 
+    fn description(&self) -> &str {
+        self.get_acquisition().description()
+    }
+
+    fn width(&self) -> i32 {
+        self.get_acquisition().width()
+    }
+
+    fn height(&self) -> i32 {
+        self.get_acquisition().height()
+    }
+
     pub fn before_ablation_image<'py>(&self, py: Python<'py>) -> &'py PyArray3<u8> {
         let acquisition = self.get_acquisition();
 
